source("https://bioconductor.org/biocLite.R")
biocLite("dada2")

# The downloaded source packages are in
# 'C:\Users\chh\AppData\Local\Temp\RtmpgVSZIO\downloaded_packages'
# installation path not writeable, unable to update packages: boot, mgcv

require(dada2)
require(ShortRead)
require(ggplot2)
require(gridExtra)

setwd("D:/Additional_projects/HaukeKegler")
load("HaukeSeqs_Spermonde_version_1.4.Rdata")
# save.image("HaukeSeqs_Spermonde_version_1.4.Rdata")

packageVersion("dada2")
# 1.4.0

### run dada2 ####

# list files
path <- "D:/Additional_projects/HaukeKegler/Clipped/"
fns <- list.files(path)
fns

# Sort ensures forward/reverse reads are in same order
fnFs <- sort(list.files(path, pattern="_R1.fastq"))
fnRs <- sort(list.files(path, pattern="_R2.fastq"))
# Extract sample names, assuming filenames have format: SAMPLENAME_XXX.fastq
sample.names <- sapply(strsplit(fnFs, "_"), `[`, 1)
# Specify the full path to the fnFs and fnRs
fnFs <- file.path(path, fnFs)
fnRs <- file.path(path, fnRs)

# quality check
QualityProfileFs <- list()
for(i in 1:length(fnFs)) {
  QualityProfileFs[[i]] <- list()
  QualityProfileFs[[i]][[1]] <- plotQualityProfile(fnFs[i])
}
pdf("QualityProfileForward.pdf")
for(i in 1:length(fnFs)) {
  do.call("grid.arrange", QualityProfileFs[[i]])  
}
dev.off()
rm(QualityProfileFs)

QualityProfileRs <- list()
for(i in 1:length(fnRs)) {
  QualityProfileRs[[i]] <- list()
  QualityProfileRs[[i]][[1]] <- plotQualityProfile(fnRs[i])
}
pdf("QualityProfileReverse.pdf")
for(i in 1:length(fnRs)) {
  do.call("grid.arrange", QualityProfileRs[[i]])  
}
dev.off()
rm(QualityProfileRs)
# length forward: 190-220
# length reverse: 170-190
# expected length: 400-430
# min overlap: 10
# try 235 forward + 215 reverse

# Make directory and filenames for the filtered fastqs
filt_path <- file.path(path, "../Filtered")
if(!file_test("-d", filt_path)) dir.create(filt_path)
filtFs <- file.path(filt_path, paste0(sample.names, "_F_filt.fastq"))
filtRs <- file.path(filt_path, paste0(sample.names, "_R_filt.fastq"))

# Filter
# depending on the expected overlap, the filtering parameters can be stricter
out <- filterAndTrim(
  fnFs, 
  filtFs, 
  fnRs, 
  filtRs,
  truncLen=c(245, 205),
  maxN=0, maxEE=c(3,3), 
  truncQ=2, 
  rm.phix=TRUE,
  compress=F,
  multithread=F
)

summary(out[, 2]/out[, 1])
# about 40-50% of sequences discarded
# not unusual for LGC :(
# reverse read quality still quite low towards the end
# adjusted read length for filtering: 245 forward + 205 reverse

# quality check
QualityProfileFs.filt <- list()
for(i in 1:length(filtFs)) {
  QualityProfileFs.filt[[i]] <- list()
  QualityProfileFs.filt[[i]][[1]] <- plotQualityProfile(filtFs[i])
}
pdf("QualityProfileForwardFiltered.pdf")
for(i in 1:length(filtFs)) {
  do.call("grid.arrange", QualityProfileFs.filt[[i]])  
}
dev.off()
rm(QualityProfileFs.filt)

QualityProfileRs.filt <- list()
for(i in 1:length(filtRs)) {
  QualityProfileRs.filt[[i]] <- list()
  QualityProfileRs.filt[[i]][[1]] <- plotQualityProfile(filtRs[i])
}
pdf("QualityProfileReverseFiltered.pdf")
for(i in 1:length(filtRs)) {
  do.call("grid.arrange", QualityProfileRs.filt[[i]])  
}
dev.off()
rm(QualityProfileRs.filt)
# looks ok

# learn errors
errF <- learnErrors(filtFs, multithread=F)
# Convergence after  6  rounds.
# Total reads used:  701101 

errR <- learnErrors(filtRs, multithread=F)
# Convergence after  8  rounds.
# Total reads used:  701101 

# plot error profiles
plotErrors(errF, nominalQ=TRUE)
plotErrors(errR, nominalQ=TRUE)

# dereplication
derepFs <- derepFastq(filtFs, verbose=TRUE)
derepRs <- derepFastq(filtRs, verbose=TRUE)
# Name the derep-class objects by the sample names
names(derepFs) <- sample.names
names(derepRs) <- sample.names

# denoising
dadaFs <- dada(derepFs, err=errF, multithread=F, pool=TRUE)
dadaRs <- dada(derepRs, err=errR, multithread=F, pool=TRUE)

# merging
# since one read will be preferred to the other for the overlap region, allowing mismatches may introduce error again?
mergers <- mergePairs(
  dadaFs, 
  derepFs, 
  dadaRs,
  derepRs,
  verbose=TRUE,
  minOverlap = 10,
  maxMismatch = 0, 
  propagateCol = c("birth_fold", "birth_ham")
)

# create sequence table
seqtab <- makeSequenceTable(mergers)
# The sequences being tabled vary in length.

dim(seqtab)
table(nchar(colnames(seqtab)))
# ranging from 274 to 429
hist(nchar(colnames(seqtab)), breaks = 100)
# only take sequences between 380 and 430bp
seqtab2 <- seqtab[, nchar(colnames(seqtab)) %in% seq(380, 430)]

# removing chimeras
seqtab2.nochim <- removeBimeraDenovo(seqtab2, method="consensus", multithread=F, verbose=TRUE)
dim(seqtab2.nochim)
# Identified 2050 bimeras out of 6304 input sequences.
# 4254 OTUs remaining

# get summary nSeqs
getN <- function(x) sum(getUniques(x))
track <- cbind(out, sapply(dadaFs, getN), sapply(mergers, getN), rowSums(seqtab2), rowSums(seqtab2.nochim))
colnames(track) <- c("input", "filtered", "denoised", "merged", "tabled", "nonchim")
rownames(track) <- sample.names
track <- data.frame(track)
head(track)

# write output
write.table(seqtab2.nochim, "seqtab2.nochim.txt", quote = F, sep = "\t")
uniquesToFasta(seqtab2.nochim, "dada2_unique.fasta")
# submit dada2_unique.fasta to Silvangs (cluster threshold 1)


### silvangs taxonomy ####

# reading input data
TAX <- read.table(
  "dada2_silva_clasification_sorted.csv",
  h = F,
  row.names = 1,
  sep = "\t",
  stringsAsFactors = F
)
colnames(TAX) <- c("seq.ident", "seq.similarity", "lca_tax_slv")

CLSTR <- read.table(
  "dada2.clstr.parsed.4R",
  sep = "\t",
  h = F,
  stringsAsFactors = F,
  row.names = 1
)
colnames(CLSTR) <- "amplicon"

all.equal(rownames(CLSTR), rownames(TAX))

# creating taxonomy table for each amplicon in the cluster map
SEQ <- strsplit(CLSTR$amplicon, ";")
# taxonomy table should have one row for each amplicon, not just for each cluster
PATH <- data.frame(unlist(SEQ), stringsAsFactors = F)
# add cluster accession
PATH$clstr <- rep(rownames(CLSTR), sapply(SEQ, length))
# add empty columns for taxonomic and alignment information
PATH$lca_tax_slv <- NA
PATH$seq.ident <- NA
PATH$seq.similarity <- NA

# for each amplicon
for(i in 1:nrow(PATH)) {
  # select taxonomy and alignment information of corresponding cluster
  temp <- TAX[PATH$clstr[i], ]
  PATH$lca_tax_slv[i] <- temp$lca_tax_slv
  PATH$seq.ident[i] <- temp$seq.ident
  PATH$seq.similarity[i] <- temp$seq.similarity
  rm(temp)
}

PATH$amplicon <- sapply(PATH$unlist.SEQ., function(x) { strsplit(x, "_", fixed = T)[[1]][1] })
PATH <- PATH[order(as.numeric(gsub("sq", "", PATH$amplicon))), ]

# subset to classified amplicons
OTU <- data.frame(t(seqtab2.nochim))
# sequences are labeled with running number 1:2414
FASTA <- colnames(seqtab2.nochim)
rownames(OTU) <- PATH$amplicon
rownames(PATH) <- PATH$amplicon
names(FASTA) <- PATH$amplicon

# removing unwanted lineages
OTUa <- OTU[grep("Bacteria", PATH$lca_tax_slv), ]
PATHa <- PATH[grep("Bacteria", PATH$lca_tax_slv), ]
all.equal(rownames(OTUa), rownames(PATHa))
OTUb <- OTUa[-c(grep("Chloroplast", PATHa$lca_tax_slv), grep("Mitochondria", PATHa$lca_tax_slv)), ]
PATHb <- PATHa[-c(grep("Chloroplast", PATHa$lca_tax_slv), grep("Mitochondria", PATHa$lca_tax_slv)), ]
all.equal(rownames(OTUb), rownames(PATHb))

# sequence removed
summary(colSums(OTUb))
summary(colSums(OTUb)/colSums(OTU))
# ok

# parsing taxonomic path
SILVAtaxopath <- function(tax, SILVA) {
  # create matrix with nrow=number of OTUs and ncol=number of possible taxonomic levels
  output <- matrix(NA, nrow = length(tax), ncol = length(levels(SILVA$rank)))
  colnames(output) <- levels(SILVA$rank)
  
  for (i in 1:length(tax)) {
    if (length(tax[[i]]) == 6){
      output[i, c("domain","phylum","class","order","family","genus")] <- tax[[i]]
    } else {
      # this loop goes through the taxonomic path of an OTU one level at a time and assigns the name of each level to its correct rank according to SILVA
      for (j in 1:length(levels(SILVA$rank))) {
        if (paste(tax[[i]][1:j], collapse=";") %in% SILVA$path) {
          output[i, as.character(SILVA[SILVA$path == paste(tax[[i]][1:j], collapse = ";"), "rank"])] <- as.character(SILVA[SILVA$path == paste(tax[[i]][1:j], collapse = ";"), "node"])
        }
      }
    }
  }
  return(output)
} 

SILVA0 <- read.table(
  "D:/Repos/zmt_gitlab/bioinf/SILVA128_tax_slv_ssu_curated.tsv",
  sep = "\t",
  h = T
)
SILVA <- SILVA0[grep("Bacteria", SILVA0$path), ]
Tax0b_split <- strsplit(as.character(PATHb$lca_tax_slv), ";")
Tax <- SILVAtaxopath(Tax0b_split, SILVA)
Taxa <- Tax[,c("domain","phylum","class","order","family","genus")] 
rownames(Taxa) <- rownames(OTUb)
Taxb <- Taxa

Taxb[Taxb == "uncultured"] <- NA
k <- ncol(Taxb) - 1
for (i in 2:k) {
  if (sum(is.na(Taxb[, i])) > 1) {
    test <- Taxb[is.na(Taxb[, i]), ]
    for (j in 1:nrow(test)) {
      if (sum(is.na(test[j, i:(k + 1)])) == length(test[j, i:(k + 1)])) {
        test[j, i] <- paste(test[j, (i - 1)], "_unclassified", sep = "")
        test[j, (i + 1):(k + 1)] <- test[j, i]
      }
    }
    Taxb[is.na(Taxb[, i]), ] <- test
  }
  if (sum(is.na(Taxb[, i])) == 1) {
    test <- Taxb[is.na(Taxb[, i]), ]
    if (sum(is.na(test[i:(k + 1)])) == length(test[i:(k + 1)])) {
      test[i] <- paste(test[(i - 1)], "_unclassified", sep = "")
      test[(i + 1):(k + 1)] <- test[i]
    }
    Taxb[is.na(Taxb[, i]),] <- test
  }
}
Taxb[is.na(Taxb[, (k + 1)]), (k + 1)] <- paste(Taxb[is.na(Taxb[, (k + 1)]), k], "_unclassified", sep = "")

align_qual <- PATHb[rownames(OTUb), c("amplicon", "seq.similarity", "lca_tax_slv")]
otu_final <- OTUb
tax_final <- Taxb

# removing 'No Relative' OTUs
# removal of OTUs not classified on phylum level
otu_good <- otu_final[align_qual$seq.similarity >= 93 & align_qual$lca_tax_slv != "Bacteria;", ]
tax_good <- tax_final[align_qual$seq.similarity >= 93 & align_qual$lca_tax_slv != "Bacteria;", ]
align_good <- align_qual[align_qual$seq.similarity >= 93 & align_qual$lca_tax_slv != "Bacteria;", ]

# sequence removed
summary(colSums(otu_good))
summary(colSums(otu_good)/colSums(OTU))
# ok

write.table(otu_good, "dada2_table.txt", sep = "\t", quote = F)
write.table(tax_good, "dada2_taxonomy.txt", sep = "\t", quote = F)
write.table(align_good, "dada2_align_qual.txt", sep = "\t", quote = F)
# manually check dada2_taxonomy.txt for inconsistencies