# R analysis 
# Hauke Spermonda small-scale

# setwd("D:/Zmt_home/Publications_inprogress/HaukeSeqs/submission_NEW/R_analysis_dada2/")
setwd("D:/Zmt_home/Publications_inprogress/HaukeSeqs/submission_NEW/R_analysis_dada2_v1.4/")
load("Hauke_R_analysis.Rdata")
# save.image("Hauke_R_analysis.Rdata")

require(vegan)
require(reshape) 
require(lmerTest) # don't use for permutation tests and rda
require(permute)
require(iNEXT)
require(gplots)
require(mada)
require(ALDEx2)
require(car)
require(ape)
require(plyr)
require(randomForest)
require(caret)
require(rfUtilities)

# 1: reading and formatting input
# 2: PCA water parameters
# 3: GLMM water parameters
# 4: summary plot water parameters
# 5: iNEXT for nOTU and invS
# 6: alpha diversity among islands and with water quality (separate for each habitat)
# 7: taxonomic composition
# 8: beta diversity (clustering and NMDS)
# 9: RDA community composition (among islands and with water quality, separate for each habitat)
# 10: ANOSIM among islands (separate for each habitat)
# 11: common trends among fractions
# 12: Random forests
# 13: potential pathogens



### 1: reading and formatting input ####

# metadata
META0 <- read.table("../small_scale_meta.txt", h = T, sep = "\t")[, 1:5]

# environmental parameters
WATER0 <- read.table("../water_env.txt", h = T, sep = "\t")

# OTU table
OTU0 <- read.table("dada2_table.txt", h = T, sep = "\t")
OTU0 <- OTU0[, colSums(OTU0) > 10]
OTU0 <- OTU0[rowSums(OTU0) > 0, ]

# Taxonomy table
TAX0 <- read.table("dada2_taxonomy_curated.txt", h = T, sep = "\t", row.names = 1)

# format sample names
META <- META0[order(META0$Island, META0$Transect, META0$Distance), ]
data.frame(gsub(".", "_", colnames(OTU0), fixed = T), as.character(META$ID))
all.equal(gsub(".", "_", colnames(OTU0), fixed = T), as.character(META$ID))
# match (checked manually)
OTU <- OTU0
colnames(OTU) <- as.character(META$ID)
rownames(META) <- as.character(META$ID)

# check OTU names
all.equal(rownames(OTU), rownames(TAX0))
TAX <- TAX0

# format environmental and metadata
META$type <- META$Habitat
levels(META$type) <- c("FL", "PA", "SED")
META$Island <- ordered(META$Island, levels = c("Uninhabited", "Inhabited"))

WATER <- WATER0
WATER$Island <- ordered(WATER$Island, levels = c("Uninhabited", "Inhabited"))
WATER$Distance <- WATER$transect_point
levels(WATER$Distance) <- c("25", "75", "150", "300")
WATER$Distance <- as.numeric(as.character(WATER$Distance))

# remove unformatted data
rm(META0, WATER0, OTU0, TAX0)

### end reading and formatting input ####

### 2: PCA water parameters ####

# symbol: diamond
# color:
#   inhabited south - blue
#   inhabited north - red
#   uninhabited south - orange
#   uninhabited north - green

WATER$color <- interaction(WATER$Island, WATER$Transect)
levels(WATER$color) <- c("green", "red", "orange", "blue")
WATER$color <- as.character(WATER$color)
WATER$cex <- sqrt(as.numeric(WATER$transect_point))
WATER$pch <- c(23)

pca_WATER <- rda(WATER[, 5:10], scale = T)
pca_WATER.plot <- plot(pca_WATER, scaling = 2)

pdf("PCA_plot.pdf", width = 7, height = 7)
plot(pca_WATER.plot$sites[, 1:2], 
     type = "n",
     las = 1,
     asp = 1,
     xlab = paste("PC1 (", round(pca_WATER$CA$eig[1]/sum(pca_WATER$CA$eig) * 100, 2), "%)", sep = ""),
     ylab = paste("PC2 (", round(pca_WATER$CA$eig[2]/sum(pca_WATER$CA$eig) * 100, 2), "%)", sep = ""),
     lwd = 0.5)
abline(h = 0, lty = 3, lwd = 0.5)
abline(v = 0, lty = 3, lwd = 0.5)
points(pca_WATER.plot$sites[, 1:2], 
       bg = WATER$color,
       col = "black", 
       pch = WATER$pch,
       cex = WATER$cex,
       lwd = 0.5)
for(i in 1:6) {
  arrows(0, 0, pca_WATER.plot$species[i, 1], pca_WATER.plot$species[i, 2],
         col = "black",
         length = 0.1, angle = 20, lwd = 0.5)
  text(pca_WATER.plot$species[i, 1],
       pca_WATER.plot$species[i, 2] + 0.05,
       labels = c(
         expression('NO'["x"]^"-"),
         expression('PO'[4]^"3-"),
         "Si",
         "Chl a",
         "DOC",
         "TEP"
       )[i], 
       cex=1, 
       pos=2, 
       font=3)
}
# plot.new()
legend('topright', 
       legend = c("Island",
                  "Inhabited - North", 
                  "Inhabited - South", 
                  "Uninhabited - North",
                  "Uninhabited - South",
                  "",
                  "Distance",
                  "25 m",
                  "75 m",
                  "150 m",
                  "300 m"),
       pch = 23, pt.lwd = 0.5, bg = "white",
       pt.bg = c(NA, "red", "blue", "green", "orange", NA, NA, "black", "black", "black", "black"),
       col = c(NA, rep("black", 4), NA, NA, "black", "black", "black", "black"),
       pt.cex = c(NA, rep(1.5, 4), NA, NA, unique(WATER$cex)))
dev.off()

### end PCA water parameters ####

### 3: GLMM water parameters ####

mod1 <- lmer(log(NOx) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod1)
shapiro.test(residuals(mod1)) 
aov1 <- anova(mod1)
plot(log(WATER$NOx) ~ WATER$Distance, col = as.numeric(WATER$Island), pch = 16)

mod2 <- lmer(log(PO4) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod2)
shapiro.test(residuals(mod2)) 
aov2 <- anova(mod2)
plot(log(WATER$PO4) ~ WATER$Distance, col = as.numeric(WATER$Island), pch = 16)

mod3 <- lmer(log(Si) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod3)
shapiro.test(residuals(mod3))
aov3 <- anova(mod3)
plot(log(WATER$Si) ~ WATER$Distance, col = as.numeric(WATER$Island), pch = 16)

mod4 <- lmer(log(Chlorophyll.a + 1) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod4)
shapiro.test(residuals(mod4)) 
aov4 <- anova(mod4)
plot(log(WATER$Chlorophyll.a + 1) ~ WATER$Distance, col = as.numeric(WATER$Island), pch = 16)

mod5 <- lmer(log(DOC) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod5)
shapiro.test(residuals(mod5))
aov5 <- anova(mod5)
plot(log(WATER$DOC) ~ WATER$Distance, col = as.numeric(WATER$Island), pch = 16)

mod6 <- lmer(log(TEP) ~ Island * Distance + (1|Transect), data = WATER)
plot(mod6)
shapiro.test(residuals(mod6)) 
aov6 <- anova(mod6)
plot(log(WATER$TEP) ~ as.numeric(WATER$Distance), col = as.numeric(WATER$Island), pch = 16)

aov.glmm <- list(aov1,aov2,aov3,aov4,aov5,aov6)
output.glmm <- matrix(NA, 6, 15)
rownames(output.glmm) <- colnames(WATER[, 5:10])
colnames(output.glmm) <- c(
  paste("Main1", c("SumSq", "F", "dfn", "dfd", "p"), sep = "."),
  paste("Main2", c("SumSq", "F", "dfn", "dfd", "p"), sep = "."),
  paste("Int", c("SumSq", "F", "dfn", "dfd", "p"), sep = ".")
)

for (i in 1:6) {
  output.glmm[i, 1:5] <- c(aov.glmm[[i]]$'Sum Sq'[1],
                           aov.glmm[[i]]$'F.value'[1],
                           aov.glmm[[i]]$'NumDF'[1],
                           aov.glmm[[i]]$'DenDF'[1],
                           aov.glmm[[i]]$'Pr(>F)'[1])
  output.glmm[i, 6:10] <- c(aov.glmm[[i]]$'Sum Sq'[2],
                            aov.glmm[[i]]$'F.value'[2],
                            aov.glmm[[i]]$'NumDF'[2],
                            aov.glmm[[i]]$'DenDF'[2],
                            aov.glmm[[i]]$'Pr(>F)'[2])
  output.glmm[i, 11:15] <- c(aov.glmm[[i]]$'Sum Sq'[3],
                             aov.glmm[[i]]$'F.value'[3],
                             aov.glmm[[i]]$'NumDF'[3],
                             aov.glmm[[i]]$'DenDF'[3],
                             aov.glmm[[i]]$'Pr(>F)'[3])
}

# run permutation test
detach("package:lmerTest", unload=TRUE)

glmm.perm <- output.glmm
for(j in rownames(glmm.perm)) {
  if (sum(WATER[, j] == 0) > 0) {
    Y <- log(WATER[, j] + 1)
  } else {
    Y <- log(WATER[, j])
  }
  Fr <- matrix(NA, 999, 3)
  for(i in 1:999) {
    modr <- lmer(Y[shuffle(length(Y))] ~ Island * Distance + (1|Transect), data = WATER)
    aovr <- anova(modr)
    Fr[i, ] <- aovr$'F value'
  }
  glmm.perm[j, 5] <- sum(Fr[,1] > glmm.perm[j, 2])/1000
  glmm.perm[j, 10] <- sum(Fr[,2] > glmm.perm[j, 7])/1000 
  glmm.perm[j, 15] <- sum(Fr[,3] > glmm.perm[j, 12])/1000
  
}

write.table(round(output.glmm, 3), "output.glmm.txt", quote = F, sep = "\t")
write.table(round(glmm.perm, 3), "glmm.perm.txt", quote = F, sep = "\t")

### end GLMM water parameters ####

### 4: summary plot water parameters ####

# plot water parameters mean +- sd

# calculate means and sd
X.melt <- melt(WATER)
head(X.melt) 
input.mean <- cast(X.melt, formula = Island + Transect + transect_point + color ~ variable, fun.aggregate = mean, na.rm = T) 
input.sd <- cast(X.melt, formula = Island + Transect + transect_point + color ~ variable, fun.aggregate = sd, na.rm = T) 
input.mean.water <- input.mean[, 5:10]
input.sd.water <- input.sd[, 5:10]

pdf("Water_quality.pdf", width = 7, height = 8)
par(
  mfrow = c(3, 2),
  mar = c(1, 5, 1, 1),
  oma = c(4, 0, 0, 0),
  xpd = NA,
  lwd = 0.5
)
for (i in 1:ncol(input.mean.water)) {
  plot(
    0, 0, # since no data is plotted yet, x and y coordinates are irrelevant
    type = "n", # don't plot anything
    xlim = c(0, 325),
    ylim = c(min(input.mean.water[, i] - input.sd.water[, i]), max(input.mean.water[, i] + input.sd.water[, i])), 
    xlab = "", # don't plot x-axis label
    ylab = c(
      expression('NO'["x"]^"-"*' [mol L'^-1*']'),
      expression('PO'[4]^"3-"*' [mol L'^-1*']'),
      expression('Si [mol L'^-1*']'),
      expression('Chl a [g L'^-1*']'),
      expression('DOC [mol L'^-1*']'),
      expression('TEP [g X'["eq"]*' L'^-1*']')
    )[i], 
    axes = F
  )
  
  if(i %in% c(5, 6)) {
    axis(1, las = 1, lwd = 0.5)
  } else {
    axis(1, las = 1, lwd = 0.5, labels = F)
  }
  axis(2, las = 1, lwd = 0.5)
  
  box(
    which = "plot", 
    lty = "solid",
    lwd = 0.5
  )
  
  text(
    -75,
    par("usr")[4],
    labels = c("(A)", "(B)", "(C)", "(D)", "(E)", "(F)")[i],
    font = 2
  )
  
  segments(
    lwd = 0.5,
    x0 = input.mean$Distance, # draw lines from x ...
    y0 = input.mean.water[, i] - input.sd.water[, i], # ... and from y ...
    x1 = input.mean$Distance, # ... to x ...
    y1 = input.mean.water[, i] + input.sd.water[, i] # ... and y
  )
  segments(
    lwd = 0.5,
    x0 = input.mean$Distance - 5, 
    y0 = input.mean.water[, i] - input.sd.water[, i], 
    x1 = input.mean$Distance + 5, 
    y1 = input.mean.water[, i] - input.sd.water[, i]
  )
  segments(
    lwd = 0.5,
    x0 = input.mean$Distance - 5, 
    y0 = input.mean.water[, i] + input.sd.water[, i], 
    x1 = input.mean$Distance + 5, 
    y1 = input.mean.water[, i] + input.sd.water[, i]
  )
  
  points(
    lwd = 0.5,
    input.mean$Distance, # x values
    input.mean.water[, i], # y values
    pch = input.mean$pch, # point character (symbol)
    cex = 1, # character expansion (point size)
    bg = input.mean$color, # color
    col = "black"
  )
  if (i == 1) {
    legend('topright', 
           legend = c("Inhabited - North", 
                      "Inhabited - South", 
                      "Uninhabited - North",
                      "Uninhabited - South"),
           pch = 23,
           pt.bg = c("red", "blue", "green", "orange"),
           col = c(rep("black", 4)),
           pt.cex = c(rep(1, 4)),
           pt.lwd = 0.5)
  }
}
mtext(
  text = "Distance to island [m]", 
  line = 2,
  side = 1, 
  outer = T
)
dev.off()

ENV <- data.frame(
  input.mean,
  cast(META, "Island + Transect + Distance ~ type", value = "ID")[, 4:6]
)

### end summary plot water parameters ####

### 5: iNEXT for nOTU and invS ####

inext.OTU.FL <- iNEXT(
  OTU[, META$type == "FL"], 
  q = c(0, 2), # Hill number
  datatype = "abundance",
  size = NULL,
  endpoint = 20000,
  knots = 80,
  se = TRUE,
  conf = 0.95,
  nboot = 100
)

inext.OTU.PA <- iNEXT(
  OTU[, META$type == "PA"], 
  q = c(0, 2), # Hill number
  datatype = "abundance",
  size = NULL,
  endpoint = 20000,
  knots = 80,
  se = TRUE,
  conf = 0.95,
  nboot = 100
)

inext.OTU.SED <- iNEXT(
  OTU[, META$type == "SED"],
  q = c(0, 2), # Hill number
  datatype = "abundance",
  size = NULL,
  endpoint = 20000,
  knots = 80,
  se = TRUE,
  conf = 0.95,
  nboot = 100
)

source("D:/Repos/zmt_gitlab/HaukeKegler_Spermonde_Illumina/iNEXTplot_custom.R")

pdf("iNEXT_plot.pdf", width = 7, height = 7, onefile = T)
layout(
  mat = matrix(c(1, 2, 3, 4, 5, 6, 7, 8, 9), 3, 3, byrow = T),
  widths = c(2, 2, 1)
)
par(mar = c(3, 4, 2, 0), mgp = c(2, 0.5, 0))
# FL
iNEXTplot(
  inext.OTU.FL, 
  hill = c(0, 2),
  color = ENV$color[match(names(inext.OTU.FL$iNextEst), ENV$FL)], 
  window.new = F,
  dynamic.ylim = T,
  xlim = c(0, 20000),
  panel.no = c("(A)", "(B)"),
  legend = F
)
# spacer
plot.new()
# PA
iNEXTplot(
  inext.OTU.PA, 
  hill = c(0, 2), 
  color = ENV$color[match(names(inext.OTU.PA$iNextEst), ENV$PA)],
  window.new = F,
  xlim = c(0, 20000),
  dynamic.ylim = T,
  panel.no = c("(C)", "(D)"),
  legend = F
)
# spacer
plot.new()
# SED
iNEXTplot(
  inext.OTU.SED, 
  hill = c(0, 2), 
  color = ENV$color[match(names(inext.OTU.SED$iNextEst), ENV$SED)], 
  window.new = F,
  xlim = c(0, 20000),
  dynamic.ylim = T,
  panel.no = c("(E)", "(F)"),
  legend = F
)
# legend
par(mar = c(4, 0, 2, 0))
plot(
  0, 0,
  type = "n",
  axes = F,
  xlim = c(0, 7),
  ylim = c(4, 1),
  xlab = "",
  ylab = ""
)
points(
  x = rep(0.5, 4),
  y = 1:4,
  cex = 2,
  pch = 15,
  col = adjustcolor(c("red", "blue", "green", "orange"), alpha.f = 0.3)
)
segments(
  x0 = rep(0, 4),
  y0 = 1:4,
  x1 = rep(1, 4),
  y1 = 1:4,
  lty = 1,
  col = c("red", "blue", "green", "orange")
)
text(
  x = rep(1, 4),
  y = 1:4,
  pos = 4,
  labels = c("Inhabited - North", "Inhabited - South", "Uninhabited - North", "Uninhabited - South"),
  cex = 0.8
)
dev.off()

### end iNEXT for nOTU and invS ####

### 6: alpha diversity among islands and with water quality ####

# calculate alpha diversity for equal sequencing depths
source("D:/Repos/chassenr/NGS/Plotting/SubsampleNGS.R")
Alpha <- SubsampleNGS(OTU, n = 100, sub = min(colSums(OTU)))

plot(Alpha$summaryHill[3, ] ~ Alpha$summaryHillRaw[3, ])
abline(0, 1)
source("D:/Repos/chassenr/NGS/Plotting/PlotHillNGS.R")
PlotHillNGS(Alpha, per.sample = F, cond = META[colnames(Alpha$summaryHillRaw), "type"], raw = T, color = 1:3)
# subsampling still necessary
# sediment more diverse than FL and PA
# FL more diverse than PA

# differences among islands
invS.summary <- cast(
  data.frame(
    META[colnames(Alpha$summaryHill), ],
    invS = Alpha$summaryHill[3, ]
  ),
  "Island + Transect + Distance ~ type",
  value = "invS"
)
boxplot(invS.summary$FL ~ invS.summary$Island)
wilcox.test(invS.summary$FL ~ invS.summary$Island)

boxplot(invS.summary$PA ~ invS.summary$Island)
wilcox.test(invS.summary$PA ~ invS.summary$Island)

boxplot(invS.summary$SED ~ invS.summary$Island)
wilcox.test(invS.summary$SED ~ invS.summary$Island)

# correlation matrix with individual environmental parameters
invS.cor <- cor(ENV[,5:10], invS.summary[, 4:6], method = "spearman", use = "pairwise.complete.obs")
colnames(invS.cor) <- levels(META$type)

heatmap.2(
  invS.cor, 
  mar=c(5,10), 
  col=bluered(40),
  scale="none",
  key=TRUE,
  symkey=F,
  density.info="none",
  trace="none", 
  cexCol=1, 
  cexRow=1, 
  keysize=1.5, 
  dendrogram="none",
  Colv=F,
  Rowv=F,
  breaks=seq(-1,1,by=0.05)
)

# plot confidence interval of correlation coefficient
# 6 panel figure:
# for each fraction (FL, PA, SED)
# jitter plot + median per island
# color: island*transect
# cex: distance
# pch: fraction
# and correlation coefficient with CI
# for each measured water parameter

invS.cor.CI <- list()
invS.cor.CI[[1]] <- CIrho(invS.cor[, 1], 11, level = 0.95)
invS.cor.CI[[2]] <- CIrho(invS.cor[, 2], 16, level = 0.95)
invS.cor.CI[[3]] <- CIrho(invS.cor[, 3], 15, level = 0.95)
names(invS.cor.CI) <- levels(META$type)

pdf("invS_water.pdf", width = 7, height = 8)
par(
  mfrow = c(3, 2),
  mar = c(1, 5, 2, 1),
  oma = c(3, 0, 0, 0),
  lwd = 0.5
)

for(i in 1:3) {
  # first plot: jitterplot
  par(xpd = NA)
  plot(
    0, 0,
    type = "n", # don't plot anything
    xlim = c(0.5, 2.5),
    ylim = if(i == 1) { c(0, 70) } else { c(0, max(invS.summary[, 3 + i], na.rm = T)) }, 
    xlab = "", # don't plot x-axis label
    ylab = paste("Inverse Simpson Index (", levels(META$type)[i], ")", sep = ""), 
    axes = F
  )
  
  # add axes
  if(i == 3) {
    axis(1, las = 1, lwd = 0.5, at = 1:2, labels = levels(META$Island))
  } else {
    axis(1, las = 1, lwd = 0.5, at = 1:2, labels = F)
  }
  axis(2, las = 1, lwd = 0.5)
  
  # add box around plotting area
  box(
    which = "plot", 
    lty = "solid",
    lwd = 0.5
  )
  
  # plot median
  segments(
    lwd = 0.5,
    x0 = 1:2 - 0.3, # draw lines from x ...
    y0 = c(by(invS.summary[, i + 3], invS.summary$Island, median, na.rm = T)), # ... and from y ...
    x1 = 1:2 + 0.3, # ... to x ...
    y1 = c(by(invS.summary[, i + 3], invS.summary$Island, median, na.rm = T)) # ... and y
  )
  
  # plot individual data points
  points(
    jitter(as.numeric(invS.summary$Island)),
    invS.summary[, i + 3],
    bg = ENV$color,
    cex = ENV$cex,
    pch = c(25, 24, 22)[i]
  )
  
  # add panel number
  text(
    par("usr")[1] - abs(par("usr")[1]) * 0.3,
    par("usr")[4] * 1.05,
    labels = c("(A)", "(C)", "(E)")[i],
    font = 2,
    cex = 1.5
  )
  
  # add axis title
  if(i == 3) {
    mtext(
      text = "Island", 
      line = 3,
      side = 1, 
      outer = F
    )
  }
  
  # legend
  if(i == 1) {
    legend(x = c(par("usr")[1], 1.5), 
           y = c(par("usr")[4], 45),
           legend = c("Island",
                      "Inhabited - North", 
                      "Inhabited - South", 
                      "Uninhabited - North",
                      "Uninhabited - South"),
           pch = 18,
           col = c(NA, "red", "blue", "green", "orange"),
           pt.cex = c(NA, rep(1.5, 4)))
  }
  if(i == 2) {
    legend(x = c(par("usr")[1], 1.5), 
           y = c(par("usr")[4], 13), 
           legend = c("Distance",
                      "25 m",
                      "75 m",
                      "150 m",
                      "300 m"),
           pch = 18,
           col = c(NA, "black", "black", "black", "black"),
           pt.cex = c(NA, unique(ENV$cex)))
  }
  
  # second plot: rho + CI
  par(xpd = F)
  plot(0,0,
       type = "n",
       ylim = c(0.5, 6.5),
       xlim = c(-1, 1),
       axes = F,
       ylab = "",
       xlab = "",
       yaxs = "i")
  if(i == 3) {
    axis(1, lwd = 0.5)
  } else {
    axis(1, lwd = 0.5, labels = F)
  }
  abline(v = 0)
  
  # add cI
  segments(invS.cor.CI[[i]][, 2],
           1:6,
           invS.cor.CI[[i]][, 3],
           1:6)
  segments(invS.cor.CI[[i]][, 2],
           1:6 - 0.3,
           invS.cor.CI[[i]][, 2],
           1:6 + 0.3)
  segments(invS.cor.CI[[i]][, 3],
           1:6 - 0.3,
           invS.cor.CI[[i]][, 3],
           1:6 + 0.3)
  
  # add rho
  points(invS.cor.CI[[i]][, 1],
         1:6,
         cex = 2,
         pch = 16)
  
  # add environmental parameter names
  par(xpd = NA)
  text(
    rep(-1, 6),
    1:6,
    labels = c(
      expression('NO'["x"]^"-"),
      expression('PO'[4]^"3-"),
      "Si",
      "Chl a",
      "DOC",
      "TEP"
    ),
    pos = 2
  )
  
  # add panel number
  text(
    par("usr")[1] - abs(par("usr")[1]) * 0.3,
    par("usr")[4] * 1.05,
    labels = c("(B)", "(D)", "(F)")[i],
    font = 2,
    cex = 1.5
  )
  
  # add axis title
  if(i == 3) {
    mtext(
      text = "Spearman's rho", 
      line = 3,
      side = 1, 
      outer = F
    )
  }
}
dev.off()

### end alpha diversity among islands and with water quality ####

### 7: taxonomic composition ####

# pool counts per taxon
TAX.pooled <- vector(
  mode = "list", # type of vector
  length = 5 # length of list, i.e. how many different taxonomic levels
)
names(TAX.pooled) <- colnames(TAX)[2:6] # phylum to genus

# fill elements of list with data frames with taxon abundance, 
# one taxonomic level at a time
for (i in 1:5) { # curly brackets are used to define what is done for each iteration of the loop
  temp <- aggregate(
    OTU, # take sequence counts from OTU
    by = list(TAX[, i + 1]), # for each taxon of the taxonomic level i + 1 (we don't want domain, which is in column 1)
    FUN = sum # sum up sequence counts
  )
  rownames(temp) <- temp$Group.1 # name rows based on taxon names
  TAX.pooled[[i]] <- temp[, -1] # since rows contain taxon names, Group.1 variable not needed anymore as part of the data frame
  rm(temp) # remove temporary object
}
TAX.pooled.rel <- lapply( # 'list apply'
  TAX.pooled, # apply function to each element of TAX.pooled
  function(x) { # define the function to be applied
    prop.table(as.matrix(x), 2) * 100 # same function as above, x is a placeholder for each element in TAX.pooled
  }
)

# average per island and transect
OTU.sum <- aggregate(
  t(OTU),
  by = list(interaction(META$Island, META$type, sep = " ")),
  FUN = sum
)
rownames(OTU.sum) <- OTU.sum$Group.1
OTU.sum <- t(OTU.sum[,-1])
all.equal(rownames(OTU.sum), rownames(TAX))
TAX.sum.pooled <- vector(
  mode = "list", # type of vector
  length = 5 # length of list, i.e. how many different taxonomic levels
)
names(TAX.sum.pooled) <- colnames(TAX)[2:6] # phylum to genus
for (i in 1:5) { # curly brackets are used to define what is done for each iteration of the loop
  temp <- aggregate(
    OTU.sum, # take sequence counts from OTU
    by = list(TAX[, i + 1]), # for each taxon of the taxonomic level i + 1 (we don't want domain, which is in column 1)
    FUN = sum # sum up sequence counts
  )
  rownames(temp) <- temp$Group.1 # name rows based on taxon names
  TAX.sum.pooled[[i]] <- temp[, -1] # since rows contain taxon names, Group.1 variable not needed anymore as part of the data frame
  rm(temp) # remove temporary object
}
TAX.sum.pooled.rel <- lapply( # 'list apply'
  TAX.sum.pooled, # apply function to each element of TAX.pooled
  function(x) { # define the function to be applied
    prop.table(as.matrix(x), 2) * 100 # same function as above, x is a placeholder for each element in TAX.pooled
  }
)

source("D:/Repos/chassenr/NGS/Plotting/PlotAbund.R")
PlotAbund(TAX.pooled.rel$class[, order(META$Habitat)], 4, margin = c(12,4,4,1))
PlotAbund(TAX.sum.pooled.rel$order, 6, margin = c(12,4,4,1))
PlotAbund(TAX.sum.pooled.rel$family, 6, margin = c(12,4,4,1))
PlotAbund(TAX.sum.pooled.rel$genus, 6, margin = c(12,4,4,1))

PlotAbund <- function(relData, abund, margin = par()$mar,
                      colorPalette = c("violet", "purple", "blue", "white", "darkgreen", "yellow", "red", "darkred"),
                      plot.ratio = c(3, 1),
                      set.layout = T,
                      cex = 0.5,
                      pt.cex = 0.8,
                      lab.cex = 0.5,
                      sample.names = colnames(relData)) {
  abund_names <- c()
  for (i in 1:ncol(relData)) {
    abund_names <- unique(c(abund_names, rownames(relData)[order(relData[, i], decreasing = T)][1:abund]))
  }
  abund_rel0 <- relData[abund_names, ]
  abund_rel <- rbind(abund_rel0, 100 - colSums(abund_rel0))
  rownames(abund_rel)[nrow(abund_rel)] <- "other"
  abund_rel <- as.matrix(abund_rel)
  if(set.layout == T) {
    layout(mat = matrix(c(1, 2), 1, 2, byrow = T), widths = plot.ratio)
  }
  par(mar = margin)
  barplot(abund_rel,
          col = c(colorRampPalette(colorPalette)(nrow(abund_rel) - 1), "darkgrey"),
          ylim = c(0, 100), 
          las = 2,
          ylab = "Relative sequence abundance [%]",
          names.arg = sample.names,
          cex.names = cex,
          cex.lab = lab.cex,
          axes = F)
  axis(2, cex.axis = cex, lwd = 0.5, las = 2, tcl = -0.3)
  par(mar = c(margin[1], 0, margin[3], 0))
  plot.new()
  legend("center", 
         pch = 22, 
         col = "black", 
         pt.bg = rev(c(colorRampPalette(colorPalette)(nrow(abund_rel) - 1), "darkgrey")),
         legend = rev(rownames(abund_rel)),
         pt.cex = pt.cex, 
         cex = cex,
         bty = "n"
  )
}

pdf("tax_comp.pdf", width = 3.4, height = 3)
par(mar = c(4, 2.5, 1, 0), cex = 0.5, lwd = 0.5, mgp = c(1.5, 0.5, 0))
PlotAbund(
  TAX.sum.pooled.rel$class, 
  5,
  plot.ratio = c(1, 1)
)
dev.off()

### end taxonomic composition ####

### 8: beta diversity ####

# cluster diagram
# if possible ordered by habitat
OTU.rel <- prop.table(as.matrix(OTU), 2) * 100
hclust.OTU <- hclust(vegdist(t(OTU.rel)), method = "average")
hclust.OTU$labels <- paste(META$Island, "_", META$Transect, "_", META$Distance, "m_", META$type, sep = "")
dendro.OTU <- as.dendrogram(hclust.OTU)
dendro.OTU <- reorder(dendro.OTU, wts = as.numeric(META$type), agglo.FUN = mean)

# NMDS plot 
# separate plots for each habitat
OTU.FL <- OTU[, META$type == "FL"]
OTU.FL <- OTU.FL[rowSums(OTU.FL) > 0, ]
OTU.PA <- OTU[, META$type == "PA"]
OTU.PA <- OTU.PA[rowSums(OTU.PA) > 0, ]
OTU.SED <- OTU[, META$type == "SED"]
OTU.SED <- OTU.SED[rowSums(OTU.SED) > 0, ]

OTU.FL.rel <- OTU.rel[, META$type == "FL"]
OTU.FL.rel <- OTU.FL.rel[rowSums(OTU.FL.rel) > 0, ]
OTU.PA.rel <- OTU.rel[, META$type == "PA"]
OTU.PA.rel <- OTU.PA.rel[rowSums(OTU.PA.rel) > 0, ]
OTU.SED.rel <- OTU.rel[, META$type == "SED"]
OTU.SED.rel <- OTU.SED.rel[rowSums(OTU.SED.rel) > 0, ]

nmds.OTU.FL <- metaMDS(t(OTU.FL.rel), trymax = 50)
envfit.OTU.FL <- envfit(nmds.OTU.FL, ENV[match(colnames(OTU.FL.rel), ENV$FL), 5:10])
nmds.OTU.PA <- metaMDS(t(OTU.PA.rel), trymax = 50)
envfit.OTU.PA <- envfit(nmds.OTU.PA, ENV[match(colnames(OTU.PA.rel), ENV$PA), 5:10])
nmds.OTU.SED <- metaMDS(t(OTU.SED.rel), trymax = 50)
envfit.OTU.SED <- envfit(nmds.OTU.SED, ENV[match(colnames(OTU.SED.rel), ENV$SED), 5:10])

# plotting cluster diagram and taxonomic composition
pdf("cluster_taxcomp.pdf", width = 7, height = 5)
layout(
  mat = matrix(c(1, 1, 1, 2, 3, 3, 3, 4), 2, 4, byrow = T),
  widths = c(1.7, 1.7, 1.7, 1),
  heights = c(1.2, 0.8)
)

# plot 1: cluster diagram
par(mar = c(12, 4, 2.5, 0), cex = 0.5, lwd = 0.5)
plot(
  dendro.OTU,
  las = 2, 
  ylab = "Bray-Curtis dissimilarity",
  axes = F,
  cex.lab = 1
)
axis(2, cex.axis = 0.8, lwd = 0.5, las = 2, tcl = -0.3)
par(xpd = NA)
text(
  rep(par("usr")[1] - abs(par("usr")[2]) * 0.05, 2),
  c(par("usr")[4] * 1.1, - 0.85),
  labels = c("(A)", "(B)"),
  font = 2,
  cex = 1.5
)
par(xpd = F)

# plot 2: spacer
plot.new()

# plot 3 and 4: taxonomic composition
par(mar = c(2, 4, 2, 0), cex = 0.5, lwd = 0.5)
PlotAbund(
  TAX.pooled.rel$class[, order.dendrogram(dendro.OTU)], 
  4,
  set.layout = F,
  cex = 0.8,
  pt.cex = 1,
  lab.cex = 1,
  sample.names = rep("", ncol(OTU))
)
dev.off()

# plotting NMDS
pdf("nmds.pdf", width = 7, height = 7)
par(
  mar = c(2.5, 3, 1.5, 1),
  mfrow = c(2, 2),
  mgp = c(1.5, 0.5, 0)
)

# plot 1: NMDS FL
plot(nmds.OTU.FL$points[, 1:2], 
     type = "n", 
     ylab = "NMDS 2", 
     xlab = "",
     las = 2,
     asp = 1,
     cex.lab = 1,
     axes = F)
axis(1, cex.axis = 0.8, lwd = 0.5, las = 1, tcl = -0.3)
axis(2, cex.axis = 0.8, lwd = 0.5, las = 2, tcl = -0.3)
box(
  which = "plot", 
  lty = "solid",
  lwd = 0.5
)
abline(v = 0, lty = 3)
abline(h = 0, lty = 3)
points(nmds.OTU.FL$points[, 1:2], 
       bg = ENV$color[match(colnames(OTU.FL.rel), ENV$FL)], 
       pch = 25, 
       cex = ENV$cex[match(colnames(OTU.FL.rel), ENV$FL)],
       col = "black")
plot(
  envfit.OTU.FL, 
  col = "black", 
  labels = c(
    expression('NO'["x"]^"-"),
    expression('PO'[4]^"3-"),
    "Si",
    "Chl a",
    "DOC",
    "TEP"
  ), 
  cex = 0.8
)
par(xpd = NA)
text(
  par("usr")[1] - abs(par("usr")[2]) * 0.1,
  par("usr")[4] * 1.1,
  labels = "(A)",
  font = 2,
  cex = 1.5
)
par(xpd = F)

# plot 2: NMDS PA
plot(nmds.OTU.PA$points[, 1:2], 
     type = "n", 
     xlab = "NMDS 1",
     ylab = "",
     las = 2,
     asp = 1,
     cex.lab = 1,
     axes = F)
axis(1, cex.axis = 0.8, lwd = 0.5, las = 1, tcl = -0.3)
axis(2, cex.axis = 0.8, lwd = 0.5, las = 2, tcl = -0.3)
box(
  which = "plot", 
  lty = "solid",
  lwd = 0.5
)
abline(v = 0, lty = 3)
abline(h = 0, lty = 3)
points(nmds.OTU.PA$points[, 1:2], 
       bg = ENV$color[match(colnames(OTU.PA.rel), ENV$PA)], 
       pch = 24, 
       cex = ENV$cex[match(colnames(OTU.PA.rel), ENV$PA)],
       col = "black")
plot(
  envfit.OTU.PA, 
  col = "black", 
  labels = c(
    expression('NO'["x"]^"-"),
    expression('PO'[4]^"3-"),
    "Si",
    "Chl a",
    "DOC",
    "TEP"
  ), 
  cex = 0.8
)
par(xpd = NA)
text(
  par("usr")[1] - abs(par("usr")[2]) * 0.1,
  par("usr")[4] * 1.1,
  labels = "(B)",
  font = 2,
  cex = 1.5
)
par(xpd = F)

# plot 7: NMDS SED
plot(nmds.OTU.SED$points[, 1:2], 
     type = "n", 
     ylab = "NMDS 2", 
     xlab = "NMDS 1",
     las = 2,
     asp = 1,
     cex.lab = 1,
     axes = F)
axis(1, cex.axis = 0.8, lwd = 0.5, las = 1, tcl = -0.3)
axis(2, cex.axis = 0.8, lwd = 0.5, las = 2, tcl = -0.3)
box(
  which = "plot", 
  lty = "solid",
  lwd = 0.5
)
abline(v = 0, lty = 3)
abline(h = 0, lty = 3)
points(nmds.OTU.SED$points[, 1:2], 
       bg = ENV$color[match(colnames(OTU.SED.rel), ENV$SED)], 
       pch = 22, 
       cex = ENV$cex[match(colnames(OTU.SED.rel), ENV$SED)],
       col = "black")
plot(
  envfit.OTU.SED, 
  col = "black", 
  labels = c(
    expression('NO'["x"]^"-"),
    expression('PO'[4]^"3-"),
    "Si",
    "Chl a",
    "DOC",
    "TEP"
  ), 
  cex = 0.8
)
par(xpd = NA)
text(
  par("usr")[1] - abs(par("usr")[2]) * 0.1,
  par("usr")[4] * 1.1,
  labels = "(C)",
  font = 2,
  cex = 1.5
)
par(xpd = F)

# plot 4: legend
plot.new()
legend('center', 
       legend = c("Habitat",
                  "FL",
                  "PA",
                  "SED",
                  "",
                  "Island",
                  "Inhabited - North", 
                  "Inhabited - South", 
                  "Uninhabited - North",
                  "Uninhabited - South",
                  "",
                  "Distance",
                  "25 m",
                  "75 m",
                  "150 m",
                  "300 m"),
       pch = c(18, 6, 2, 0, 18, rep(18, 11)), 
       pt.lwd = 1,
       bty = "n",
       col = c(NA, "black", "black", "black", NA, NA, "red", "blue", "green", "orange", NA, NA, "black", "black", "black", "black"),
       pt.cex = c(NA, rep(1.5, 4), NA, rep(1.5, 4), NA, NA, unique(WATER$cex)))

dev.off()

### end beta diversity ####

### 9: RDA community composition ####

# clr transformation using aldex.clr
# exclude OTUs that only occur in one sample per island
OTU.FL.pruned <- OTU.FL[apply(OTU.FL, 1, function(x) { max(table(ENV$Island[match(colnames(OTU.FL), ENV$FL)][x > 0])) >=2 } ), ]
OTU.FL.clr.mc <- aldex.clr(OTU.FL.pruned, mc.samples = 128)
OTU.FL.clr <- sapply(getMonteCarloInstances(OTU.FL.clr.mc), function(x) {apply(x, 1, median)})

OTU.PA.pruned <- OTU.PA[apply(OTU.PA, 1, function(x) { max(table(ENV$Island[match(colnames(OTU.PA), ENV$PA)][x > 0])) >=2 } ), ]
OTU.PA.clr.mc <- aldex.clr(OTU.PA.pruned, mc.samples = 128)
OTU.PA.clr <- sapply(getMonteCarloInstances(OTU.PA.clr.mc), function(x) {apply(x, 1, median)})

OTU.SED.pruned <- OTU.SED[apply(OTU.SED, 1, function(x) { max(table(ENV$Island[match(colnames(OTU.SED), ENV$SED)][x > 0])) >=2 } ), ]
OTU.SED.clr.mc <- aldex.clr(OTU.SED.pruned, mc.samples = 128)
OTU.SED.clr <- sapply(getMonteCarloInstances(OTU.SED.clr.mc), function(x) {apply(x, 1, median)})

ENV.FL <- ENV[match(colnames(OTU.FL.clr), ENV$FL), ]
ENV.PA <- ENV[match(colnames(OTU.PA.clr), ENV$PA), ]
ENV.SED <- ENV[match(colnames(OTU.SED.clr), ENV$SED), ]

# assess effect of pruning on beta diversity patterns
plot(
  vegdist(t(OTU.FL.rel)),
  vegdist(t(OTU.FL.rel[rownames(OTU.FL.pruned), ]))
)
abline(0, 1)
summary(colSums(OTU.FL.pruned)/colSums(OTU.FL))

plot(
  vegdist(t(OTU.PA.rel)),
  vegdist(t(OTU.PA.rel[rownames(OTU.PA.pruned), ]))
)
abline(0, 1)
summary(colSums(OTU.PA.pruned)/colSums(OTU.PA))

plot(
  vegdist(t(OTU.SED.rel)),
  vegdist(t(OTU.SED.rel[rownames(OTU.SED.pruned), ]))
)
abline(0, 1)
summary(colSums(OTU.SED.pruned)/colSums(OTU.SED))

# although there are some deviations especially for sediment communities,
# mantel statistic always > 0.99

# RDA island
rda.FL <- rda(t(OTU.FL.clr) ~ ENV.FL$Island)
ordiresids(rda.FL)
anova(rda.FL) 
RsquareAdj(rda.FL) 

rda.PA <- rda(t(OTU.PA.clr) ~ ENV.PA$Island)
ordiresids(rda.PA)
anova(rda.PA) 
RsquareAdj(rda.PA) 

rda.SED <- rda(t(OTU.SED.clr) ~ ENV.SED$Island)
ordiresids(rda.SED)
anova(rda.SED) 
RsquareAdj(rda.SED) 

# RDA water parameters
# using variance inflation factors
# also testing PCs from PCA water parameters
detach("package:lmerTest", unload=TRUE)

# FL
vif.cca(
  rda(
    t(OTU.FL.clr) ~ NOx + PO4 + Si + Chlorophyll.a + DOC + TEP,
    data = ENV.FL
  )
)

step(
  rda(t(OTU.FL.clr) ~ 1),
  scope = "t(OTU.FL.clr) ~ ENV.FL$NOx + ENV.FL$PO4 + ENV.FL$Si +
  ENV.FL$Chlorophyll.a + ENV.FL$DOC + ENV.FL$TEP",
  direction = "forward"
)
# otherwise too many predictors for small sample size
rda.FL.env <- rda(t(OTU.FL.clr) ~ Chlorophyll.a + PO4, data = ENV.FL)
ordiresids(rda.FL.env)
RsquareAdj(rda.FL.env) 
varpart.FL.env <- varpart(t(OTU.FL.clr), ENV.FL$Chlorophyll.a, ENV.FL$PO4)
plot(varpart.FL.env) 
# PO4 and Chla most important with > 15% each
anova(rda.FL.env) 
anova(rda(t(OTU.FL.clr) ~ Chlorophyll.a + Condition(PO4), data = ENV.FL)) 
anova(rda(t(OTU.FL.clr) ~ Condition(Chlorophyll.a) + PO4, data = ENV.FL)) 

# PA
vif.cca(
  rda(
    t(OTU.PA.clr) ~ NOx + PO4 + Si + Chlorophyll.a + DOC + TEP,
    data = ENV.PA
  )
)

step(
  rda(t(OTU.PA.clr) ~ 1),
  scope = "t(OTU.PA.clr) ~ ENV.PA$NOx + ENV.PA$PO4 + ENV.PA$Si +
  ENV.PA$Chlorophyll.a + ENV.PA$DOC + ENV.PA$TEP",
  direction = "forward"
)
rda.PA.env <- rda(t(OTU.PA.clr) ~ TEP + Chlorophyll.a, data = ENV.PA)
ordiresids(rda.PA.env)
RsquareAdj(rda.PA.env) 
varpart.PA.env <- varpart(t(OTU.PA.clr), ENV.PA$TEP, ENV.PA$Chlorophyll.a)
plot(varpart.PA.env) 
# both ~ 6%, 2% overlap
anova(rda.PA.env) 
anova(rda(t(OTU.PA.clr) ~ TEP + Condition(Chlorophyll.a), data = ENV.PA)) 
anova(rda(t(OTU.PA.clr) ~ Condition(TEP) + Chlorophyll.a, data = ENV.PA)) 

# SED
vif.cca(
  rda(
    t(OTU.SED.clr) ~ NOx + PO4 + Si + Chlorophyll.a + DOC + TEP,
    data = ENV.SED
  )
)

step(
  rda(t(OTU.SED.clr) ~ 1),
  scope = "t(OTU.SED.clr) ~ ENV.SED$NOx + ENV.SED$PO4 + ENV.SED$Si +
  ENV.SED$Chlorophyll.a + ENV.SED$DOC + ENV.SED$TEP",
  direction = "forward"
)
rda.SED.env <- rda(t(OTU.SED.clr) ~ Chlorophyll.a, data = ENV.SED)
ordiresids(rda.SED.env)
RsquareAdj(rda.SED.env) 
anova(rda.SED.env)

# for FL communities, Island not very good predictor
# also because of unbalanced sample size
# but individual water parameters very good
# FL directly dependent on water parameters

# for PA, water parameters not as good anymore,

# for SED, both Island and individual water parameters good predictors
# island good predictor, because there may be other factors influencing
# SED, which were not measured, but still differ among islands,
# individual water parameters not as good a predictor as for FL
# microniches?, i.e. SED not completely exposed to water conditions
# chla good predictor because it shows strongest differences between islands

### end RDA community composition ####

### 10: ANOSIM ####

# differences between habitat
anosim.type <- anosim(t(OTU.rel), META$type)
source("D:/Repos/chassenr/ARISA/anosimPosthoc.R")
anosim.type.post <- ANOSIMposthoc(t(OTU.rel), META$type)

# differences between islands
anosim.FL <- anosim(t(OTU.FL.rel), ENV.FL$Island)
anosim.PA <- anosim(t(OTU.PA.rel), ENV.PA$Island)
anosim.SED <- anosim(t(OTU.SED.rel), ENV.SED$Island)

# strong separation only for sediment
# PA weak separation but still significant
# FL weak separation, not significant (because of lower sample size)
# confirms RDA results

# betadispersion
boxplot(betadisper(vegdist(t(OTU.FL.rel)), ENV.FL$Island, type = "centroid"))
boxplot(betadisper(vegdist(t(OTU.PA.rel)), ENV.PA$Island, type = "centroid"))
boxplot(betadisper(vegdist(t(OTU.SED.rel)), ENV.SED$Island, type = "centroid"))

plot(nmds.OTU.FL, display = "sites", type = "t")
ordispider(nmds.OTU.FL, ENV.FL$Island, spiders = "centroid")
plot(nmds.OTU.PA, display = "sites", type = "t")
ordispider(nmds.OTU.PA, ENV.PA$Island, spiders = "centroid")
plot(nmds.OTU.SED, display = "sites", type = "t")
ordispider(nmds.OTU.SED, ENV.SED$Island, spiders = "centroid")

wilcox.test(betadisper(vegdist(t(OTU.FL.rel)), ENV.FL$Island, type = "centroid")$distances ~ ENV.FL$Island)
wilcox.test(betadisper(vegdist(t(OTU.PA.rel)), ENV.PA$Island, type = "centroid")$distances ~ ENV.PA$Island) 
wilcox.test(betadisper(vegdist(t(OTU.SED.rel)), ENV.SED$Island, type = "centroid")$distances ~ ENV.SED$Island)

# PA (and to lesser degree FL) more heterogeneous communities at Inhabited
# due to more heterogeneous environmental conditions?
# due to contamination from land?

# average dissimilarity 
# BC and JC per habitat between islands
mean(vegdist(t(OTU.FL.rel)[ENV.FL$Island == "Inhabited", ])) 
mean(vegdist(t(OTU.FL.rel)[ENV.FL$Island == "Uninhabited", ])) 
plot(hclust(vegdist(t(OTU.FL.rel))))
mean(as.matrix(vegdist(t(OTU.FL.rel)))[ENV.FL$Island == "Inhabited", ENV.FL$Island == "Uninhabited"]) 

mean(vegdist(t(OTU.PA.rel)[ENV.PA$Island == "Inhabited", ])) 
mean(vegdist(t(OTU.PA.rel)[ENV.PA$Island == "Uninhabited", ])) 
plot(hclust(vegdist(t(OTU.PA.rel))))
mean(as.matrix(vegdist(t(OTU.PA.rel)))[ENV.PA$Island == "Inhabited", ENV.PA$Island == "Uninhabited"]) 

mean(vegdist(t(OTU.SED.rel)[ENV.SED$Island == "Inhabited", ])) 
mean(vegdist(t(OTU.SED.rel)[ENV.SED$Island == "Uninhabited", ])) 
plot(hclust(vegdist(t(OTU.SED.rel))))
mean(as.matrix(vegdist(t(OTU.SED.rel)))[ENV.SED$Island == "Inhabited", ENV.SED$Island == "Uninhabited"]) 

### end ANOSIM ####

### 11: common trends ####

# mantel test
mantel(
  vegdist(t(OTU.FL.rel[, ENV.FL$FL[!is.na(ENV.FL$PA)]])),
  vegdist(t(OTU.PA.rel[, ENV.PA$PA[!is.na(ENV.PA$FL)]]))
)

mantel(
  vegdist(t(OTU.FL.rel[, ENV.FL$FL[!is.na(ENV.FL$SED)]])),
  vegdist(t(OTU.SED.rel[, ENV.SED$SED[!is.na(ENV.SED$FL)]]))
)

mantel(
  vegdist(t(OTU.SED.rel[, ENV.SED$SED[!is.na(ENV.SED$PA)]])),
  vegdist(t(OTU.PA.rel[, ENV.PA$PA[!is.na(ENV.PA$SED)]]))
)

# common pattern only for PA and FL
# not very strong correlation

# procrustes
plot(
  procrustes(
    metaMDS(t(OTU.FL.rel[, ENV.FL$FL[!is.na(ENV.FL$PA)]]), trymax = 50),
    metaMDS(t(OTU.PA.rel[, ENV.PA$PA[!is.na(ENV.PA$FL)]]), trymax = 50),
    symmetric = T
  )
)

plot(
  procrustes(
    metaMDS(t(OTU.FL.rel[, ENV.FL$FL[!is.na(ENV.FL$SED)]]), trymax = 50),
    metaMDS(t(OTU.SED.rel[, ENV.SED$SED[!is.na(ENV.SED$FL)]]), trymax = 50),
    symmetric = T
  )
)

plot(
  procrustes(
    metaMDS(t(OTU.SED.rel[, ENV.SED$SED[!is.na(ENV.SED$PA)]]), trymax = 50),
    metaMDS(t(OTU.PA.rel[, ENV.PA$PA[!is.na(ENV.PA$SED)]]), trymax = 50),
    symmetric = T
  )
)

# dominant patterns comparable between FL and PA
# and to lesser degree FL and SED, and PA and SED

### end common trends ####

### 12: random forests ####
# https://github.com/LangilleLab/microbiome_helper/wiki/Random-Forest-Tutorial

# FL
hist(OTU.FL.clr)
# running model
randomForest.FL <- randomForest(
  x = data.frame(t(OTU.FL.clr)),
  y = ENV.FL$Island, 
  ntree = 10001, 
  importance = TRUE, 
  proximities = TRUE
)
# model significance
randomForest.FL.sig <- rf.significance(
  x = randomForest.FL,  
  xdata = data.frame(t(OTU.FL.clr)),
  nperm = 1000,
  ntree = 10001
)
# cross-validation
RF_state_classify_loocv.FL <- train( 
  data.frame(t(OTU.FL.clr)), 
  y = ENV.FL$Island,
  method = "rf", 
  ntree = 10001 ,
  tuneGrid = data.frame(mtry = randomForest.FL$mtry),
  trControl = trainControl(method = "LOOCV") 
)
RF_state_classify_loocv.FL$results
# importance
randomForest.FL.imp <- data.frame(randomForest.FL$importance)
randomForest.FL.imp <- randomForest.FL.imp[order(randomForest.FL.imp[, "MeanDecreaseAccuracy"], decreasing = T), ]
barplot(randomForest.FL.imp$MeanDecreaseAccuracy[1:100], las = 2, cex.names = 0.8)
abline(h = 0.002)
# cut-off at 0.002
randomForest.FL.names <- rownames(randomForest.FL.imp)[randomForest.FL.imp$MeanDecreaseAccuracy > 0.002]

# randomForest.FL.imp <- randomForest.FL.imp[order(randomForest.FL.imp[, "MeanDecreaseGini"], decreasing = T), ]
# barplot(randomForest.FL.imp$MeanDecreaseGini[1:100])
# abline(h = 0.04)
# sum(randomForest.FL.imp$MeanDecreaseGini > 0.04)
# identical

# PA
hist(OTU.PA.clr)
# running model
randomForest.PA <- randomForest(
  x = data.frame(t(OTU.PA.clr)),
  y = ENV.PA$Island, 
  ntree = 10001, 
  importance = TRUE, 
  proximities = TRUE
)
# model significance
randomForest.PA.sig <- rf.significance(
  x = randomForest.PA,  
  xdata = data.frame(t(OTU.PA.clr)),
  nperm = 1000,
  ntree = 10001
)
# cross-validation
RF_state_classify_loocv.PA <- train( 
  data.frame(t(OTU.PA.clr)), 
  y = ENV.PA$Island,
  method = "rf", 
  ntree = 10001 ,
  tuneGrid = data.frame(mtry = randomForest.PA$mtry),
  trControl = trainControl(method = "LOOCV") 
)
RF_state_classify_loocv.PA$results
# importance
randomForest.PA.imp <- data.frame(randomForest.PA$importance)
randomForest.PA.imp <- randomForest.PA.imp[order(randomForest.PA.imp[, "MeanDecreaseAccuracy"], decreasing = T), ]
barplot(randomForest.PA.imp$MeanDecreaseAccuracy[1:100], las = 2, cex.names = 0.8)
abline(h = 0.003)
# cut-off at 0.003
randomForest.PA.names <- rownames(randomForest.PA.imp)[randomForest.PA.imp$MeanDecreaseAccuracy > 0.003]

# randomForest.PA.imp <- randomForest.PA.imp[order(randomForest.PA.imp[, "MeanDecreaseGini"], decreasing = T), ]
# barplot(randomForest.PA.imp$MeanDecreaseGini[1:100])
# abline(h = 0.075)
# sum(randomForest.PA.imp$MeanDecreaseGini > 0.075)
# identical

# SED
hist(OTU.SED.clr)
# running model
randomForest.SED <- randomForest(
  x = data.frame(t(OTU.SED.clr)),
  y = ENV.SED$Island, 
  ntree = 10001, 
  importance = TRUE, 
  proximities = TRUE
)
# model significance
randomForest.SED.sig <- rf.significance(
  x = randomForest.SED,  
  xdata = data.frame(t(OTU.SED.clr)),
  nperm = 1000,
  ntree = 10001
)
# cross-validation
RF_state_classify_loocv.SED <- train( 
  data.frame(t(OTU.SED.clr)), 
  y = ENV.SED$Island,
  method = "rf", 
  ntree = 10001 ,
  tuneGrid = data.frame(mtry = randomForest.SED$mtry),
  trControl = trainControl(method = "LOOCV") 
)
RF_state_classify_loocv.SED$results
# importance
randomForest.SED.imp <- data.frame(randomForest.SED$importance)
randomForest.SED.imp <- randomForest.SED.imp[order(randomForest.SED.imp[, "MeanDecreaseAccuracy"], decreasing = T), ]
barplot(randomForest.SED.imp$MeanDecreaseAccuracy[1:100], las = 2, cex.names = 0.8)
abline(h = 0.0016)
# cut-off at 0.0016
randomForest.SED.names <- rownames(randomForest.SED.imp)[randomForest.SED.imp$MeanDecreaseAccuracy > 0.0016]

# randomForest.SED.imp <- randomForest.SED.imp[order(randomForest.SED.imp[, "MeanDecreaseGini"], decreasing = T), ]
# barplot(randomForest.SED.imp$MeanDecreaseGini[1:100])
# abline(h = 0.04)
# sum(randomForest.SED.imp$MeanDecreaseGini > 0.04)
# identical

# plotting mean difference
source("D:/Repos/zmt_gitlab/HaukeKegler_Spermonde_Illumina/PlotMeanDiff_custom.R")

# FL
mean.diff.FL.rF <- matrix(
  NA,
  length(randomForest.FL.names),
  7
)
rownames(mean.diff.FL.rF) <- randomForest.FL.names
colnames(mean.diff.FL.rF) <- c("min.uninhabited", "min.inhabited", "max.uninhabited", "max.inhabited", "mean.diff", "CI.lower", "CI.upper")
mean.diff.FL.rF[, 1:2] <- t(apply(OTU.FL.rel[randomForest.FL.names, ], 1, function(x) {by(x, ENV.FL$Island, min)}))
mean.diff.FL.rF[, 3:4] <- t(apply(OTU.FL.rel[randomForest.FL.names, ], 1, function(x) {by(x, ENV.FL$Island, max)}))
mean.diff.FL.rF[, 5] <- t(apply(OTU.FL.clr[randomForest.FL.names, ], 1, function(x) { diff(t.test(x ~ ENV.FL$Island)$estimate) }))
mean.diff.FL.rF[, 6:7] <- t(apply(OTU.FL.clr[randomForest.FL.names, ], 1, function(x) {-1 * (t.test(x ~ ENV.FL$Island)$conf.int) }))
rownames(mean.diff.FL.rF) <- paste(
  gsub("_unclassified", "", as.character(TAX[randomForest.FL.names, "genus"])),
  rownames(mean.diff.FL.rF),
  sep = "_"
)
PlotMeanDiff(
  mean.diff.FL.rF,
  higher.tax = TAX[randomForest.FL.names, "class"]
)

# PA
mean.diff.PA.rF <- matrix(
  NA,
  length(randomForest.PA.names),
  7
)
rownames(mean.diff.PA.rF) <- randomForest.PA.names
colnames(mean.diff.PA.rF) <- c("min.uninhabited", "min.inhabited", "max.uninhabited", "max.inhabited", "mean.diff", "CI.lower", "CI.upper")
mean.diff.PA.rF[, 1:2] <- t(apply(OTU.PA.rel[randomForest.PA.names, ], 1, function(x) {by(x, ENV.PA$Island, min)}))
mean.diff.PA.rF[, 3:4] <- t(apply(OTU.PA.rel[randomForest.PA.names, ], 1, function(x) {by(x, ENV.PA$Island, max)}))
mean.diff.PA.rF[, 5] <- t(apply(OTU.PA.clr[randomForest.PA.names, ], 1, function(x) { diff(t.test(x ~ ENV.PA$Island)$estimate) }))
mean.diff.PA.rF[, 6:7] <- t(apply(OTU.PA.clr[randomForest.PA.names, ], 1, function(x) {-1 * (t.test(x ~ ENV.PA$Island)$conf.int) }))
rownames(mean.diff.PA.rF) <- paste(
  gsub("_unclassified", "", as.character(TAX[randomForest.PA.names, "genus"])),
  rownames(mean.diff.PA.rF),
  sep = "_"
)
PlotMeanDiff(
  mean.diff.PA.rF,
  higher.tax = TAX[randomForest.PA.names, "class"]
)

# SED
mean.diff.SED.rF <- matrix(
  NA,
  length(randomForest.SED.names),
  7
)
rownames(mean.diff.SED.rF) <- randomForest.SED.names
colnames(mean.diff.SED.rF) <- c("min.uninhabited", "min.inhabited", "max.uninhabited", "max.inhabited", "mean.diff", "CI.lower", "CI.upper")
mean.diff.SED.rF[, 1:2] <- t(apply(OTU.SED.rel[randomForest.SED.names, ], 1, function(x) {by(x, ENV.SED$Island, min)}))
mean.diff.SED.rF[, 3:4] <- t(apply(OTU.SED.rel[randomForest.SED.names, ], 1, function(x) {by(x, ENV.SED$Island, max)}))
mean.diff.SED.rF[, 5] <- t(apply(OTU.SED.clr[randomForest.SED.names, ], 1, function(x) { diff(t.test(x ~ ENV.SED$Island)$estimate) }))
mean.diff.SED.rF[, 6:7] <- t(apply(OTU.SED.clr[randomForest.SED.names, ], 1, function(x) {-1 * (t.test(x ~ ENV.SED$Island)$conf.int) }))
rownames(mean.diff.SED.rF) <- paste(
  gsub("_unclassified", "", as.character(TAX[randomForest.SED.names, "genus"])),
  rownames(mean.diff.SED.rF),
  sep = "_"
)
PlotMeanDiff(
  mean.diff.SED.rF,
  higher.tax = TAX[randomForest.SED.names, "class"],
  mar1 = c(4, 8, 1, 1)
)

# visualization as heatmap
maxValue <- max(c(c(abs(OTU.FL.clr[randomForest.FL.names, ])), 
                  c(abs(OTU.PA.clr[randomForest.PA.names, ])), 
                  c(abs(OTU.SED.clr[randomForest.SED.names, ]))))

# don't draw individual heatmaps
# combine output
# cluster prior to calling heatmap.2
temp.FL <- OTU.FL.clr[randomForest.FL.names, ]
rownames(temp.FL) <- paste(
  TAX[randomForest.FL.names, "class"],
  ": ",
  gsub("_unclassified", "", TAX[randomForest.FL.names, "genus"]),
  "_",
  randomForest.FL.names,
  sep = ""
)
temp.FL <- temp.FL[hclust(dist(temp.FL))$order, order(ENV.FL$Island)]

temp.PA <- OTU.PA.clr[randomForest.PA.names, ]
rownames(temp.PA) <- paste(
  TAX[randomForest.PA.names, "class"],
  ": ",
  gsub("_unclassified", "", TAX[randomForest.PA.names, "genus"]),
  "_",
  randomForest.PA.names,
  sep = ""
)
temp.PA <- temp.PA[hclust(dist(temp.PA))$order, order(ENV.PA$Island)]

temp.SED <- OTU.SED.clr[randomForest.SED.names, ]
rownames(temp.SED) <- paste(
  TAX[randomForest.SED.names, "class"],
  ": ",
  gsub("_unclassified", "", TAX[randomForest.SED.names, "genus"]),
  "_",
  randomForest.SED.names,
  sep = ""
)
temp.SED <- temp.SED[hclust(dist(temp.SED))$order, order(ENV.SED$Island)]

# check column names
colnames(temp.FL)
colnames(temp.PA)
colnames(temp.SED)

# combine input
input.heatmap <- rbind(
  cbind(
    matrix(NA, nrow(temp.FL), 4),
    temp.FL
  ),
  matrix(NA, 1, 16),
  temp.PA,
  matrix(NA, 1, 16),
  temp.SED
)

# draw heatmap
pdf("heatmap_random_forests.pdf", width = 7, height = 12)
heatmap.2(
  input.heatmap, 
  mar = c(10, 20), 
  col = bluered(length(seq(-max(abs(input.heatmap), na.rm = T), max(abs(input.heatmap), na.rm = T), by = 0.5)) - 1),
  scale = "none",
  key = TRUE,
  symkey = F,
  density.info = "none",
  trace = "none", 
  cexCol = 0.8, 
  cexRow = 0.7, 
  labRow = c(
    rownames(temp.FL),
    "",
    rownames(temp.PA),
    "",
    rownames(temp.SED)
  ),
  labCol = paste(ENV.PA$Island, "_", ENV.PA$Transect, "_", ENV.PA$Distance, "m", sep = "")[order(ENV.PA$Island)],
  keysize = 1,
  key.title = NA,
  key.par = list(mar = c(5, 2, 1, 0), cex = 0.6),
  key.xlab = "clr-transformed sequence counts",
  dendrogram = "none",
  Colv = F,
  Rowv = F,
  breaks = seq(-max(abs(input.heatmap), na.rm = T), max(abs(input.heatmap), na.rm = T), by = 0.5),
  lhei = c(1, 9)
)
segments(
  rep(0.09, 3),
  c(0.063, 0.486, 0.692),
  rep(0.09, 3),
  c(0.473, 0.68, 0.96),
  lwd = 1
)
text(
  rep(0.09, 3),
  apply(cbind(c(0.063, 0.486, 0.692), c(0.473, 0.68, 0.96)), 1, mean),
  labels = c("SED", "PA", "FL"),
  las = 3,
  pos = 2
)
text(
  0.15,
  0.692,
  labels = "No data",
  las = 3,
  pos = 3,
  cex = 0.8
)
dev.off()

# separate heatmaps
# FL
heatmap.2(
  OTU.FL.clr[randomForest.FL.names, ], 
  mar = c(12, 20), 
  # col = bluered(length(seq(-max(abs(OTU.FL.clr[randomForest.FL.names, ])), max(abs(OTU.FL.clr[randomForest.FL.names, ])), by = 0.5)) - 1),
  col = bluered(length(seq(-maxValue, maxValue, by = 0.5)) - 1),
  scale = "none",
  key = TRUE,
  symkey = F,
  density.info = "none",
  trace = "none", 
  cexCol = 1, 
  cexRow = 0.8, 
  labRow = paste(
    TAX[randomForest.FL.names, "class"],
    ": ",
    gsub("_unclassified", "", TAX[randomForest.FL.names, "genus"]),
    "_",
    randomForest.FL.names,
    sep = ""
  ),
  labCol = paste(ENV.FL$Island, "_", ENV.FL$Transect, "_", ENV.FL$Distance, "m", sep = ""),
  keysize = 1,
  key.title = NA,
  key.par = list(mar = c(5, 2, 1, 0), cex = 0.6),
  key.xlab = "clr-transformed sequence counts",
  dendrogram = "none",
  Colv = T,
  Rowv = T,
  # breaks = seq(-max(abs(OTU.FL.clr[randomForest.FL.names, ])), max(abs(OTU.FL.clr[randomForest.FL.names, ])), by = 0.5)
  breaks = seq(-maxValue, maxValue, by = 0.5)
)

# PA
heatmap.2(
  OTU.PA.clr[randomForest.PA.names, ], 
  mar = c(12, 20), 
  # col = bluered(length(seq(-max(abs(OTU.PA.clr[randomForest.PA.names, ])), max(abs(OTU.PA.clr[randomForest.PA.names, ])), by = 0.5)) - 1),
  col = bluered(length(seq(-maxValue, maxValue, by = 0.5)) - 1),
  scale = "none",
  key = F,
  symkey = F,
  density.info = "none",
  trace = "none", 
  cexCol = 1, 
  cexRow = 0.8, 
  labRow = paste(
    TAX[randomForest.PA.names, "class"],
    ": ",
    gsub("_unclassified", "", TAX[randomForest.PA.names, "genus"]),
    "_",
    randomForest.PA.names,
    sep = ""
  ),
  labCol = paste(ENV.PA$Island, "_", ENV.PA$Transect, "_", ENV.PA$Distance, "m", sep = ""),
  keysize = 1,
  key.title = NA,
  key.par = list(mar = c(5, 2, 1, 0), cex = 0.6),
  key.xlab = "clr-transformed sequence counts",
  dendrogram = "none",
  Colv = T,
  Rowv = T,
  # breaks = seq(-max(abs(OTU.PA.clr[randomForest.PA.names, ])), max(abs(OTU.PA.clr[randomForest.PA.names, ])), by = 0.5)
  breaks = seq(-maxValue, maxValue, by = 0.5)
)

# SED
heatmap.2(
  OTU.SED.clr[randomForest.SED.names, ], 
  mar = c(12, 20), 
  # col = bluered(length(seq(-max(abs(OTU.SED.clr[randomForest.SED.names, ])), max(abs(OTU.SED.clr[randomForest.SED.names, ])), by = 0.5)) - 1),
  col = bluered(length(seq(-maxValue, maxValue, by = 0.5)) - 1),
  scale = "none",
  key = F,
  symkey = F,
  density.info = "none",
  trace = "none", 
  cexCol = 1, 
  cexRow = 0.8, 
  labRow = paste(
    TAX[randomForest.SED.names, "class"],
    ": ",
    gsub("_unclassified", "", TAX[randomForest.SED.names, "genus"]),
    "_",
    randomForest.SED.names,
    sep = ""
  ),
  labCol = paste(ENV.SED$Island, "_", ENV.SED$Transect, "_", ENV.SED$Distance, "m", sep = ""),
  keysize = 1,
  key.title = NA,
  key.par = list(mar = c(5, 2, 1, 0), cex = 0.6),
  key.xlab = "clr-transformed sequence counts",
  dendrogram = "none",
  Colv = T,
  Rowv = T,
  # breaks = seq(-max(abs(OTU.SED.clr[randomForest.SED.names, ])), max(abs(OTU.SED.clr[randomForest.SED.names, ])), by = 0.5)
  breaks = seq(-maxValue, maxValue, by = 0.5)
)

# how much sequence contribution of random forest OTUs
boxplot(colSums(OTU.FL.rel[randomForest.FL.names, ]) ~ ENV.FL$Island)
boxplot(colSums(OTU.PA.rel[randomForest.PA.names, ]) ~ ENV.PA$Island)
boxplot(colSums(OTU.SED.rel[randomForest.SED.names, ]) ~ ENV.SED$Island)
# OTUs that differ markedly among the 2 islands  contribute quite substantially to sequence pool
# generally, most increase towards inhabited island

### end Random Forests ####

### pathogens ####
pathogens <- c("Mycobacterium", 
               "Vibrio", 
               "Ralstonia", 
               "Escherichia-Shigella",
               "Stenotrophomonas", 
               "Propionibacterium",
               "Staphylococcus",
               "Phenylobacterium")

pathogens.FL.heat <- OTU.FL.clr[TAX[rownames(OTU.FL.clr), "genus"] %in% pathogens, ]
pathogens.FL.heat <- pathogens.FL.heat[hclust(dist(pathogens.FL.heat))$order, order(ENV.FL$Island)]
range(colSums(OTU.FL.rel[rownames(pathogens.FL.heat), ]))
# 0.089035 1.854883

pathogens.PA.heat <- OTU.PA.clr[TAX[rownames(OTU.PA.clr), "genus"] %in% pathogens, ]
pathogens.PA.heat <- pathogens.PA.heat[hclust(dist(pathogens.PA.heat))$order, order(ENV.PA$Island)]
range(colSums(OTU.PA.rel[rownames(pathogens.PA.heat), ]))
# 0.3136626 3.7198173

pathogens.SED.heat <- OTU.SED.clr[TAX[rownames(OTU.SED.clr), "genus"] %in% pathogens, ]
pathogens.SED.heat <- pathogens.SED.heat[hclust(dist(pathogens.SED.heat))$order, order(ENV.SED$Island)]
range(colSums(OTU.SED.rel[rownames(pathogens.SED.heat), ]))
# 0.03603604 0.49373338

# check column names
colnames(pathogens.FL.heat)
colnames(pathogens.PA.heat)
colnames(pathogens.SED.heat)

# combine input
pathogens.heat.in <- rbind(
  cbind(
    matrix(NA, nrow(pathogens.FL.heat), 4),
    pathogens.FL.heat
  ),
  matrix(NA, 1, 16),
  pathogens.PA.heat,
  matrix(NA, 1, 16),
  pathogens.SED.heat
)

pdf("heatmap_pathogens.pdf", width = 7, height = 12)
heatmap.2(
  pathogens.heat.in, 
  mar = c(10, 10), 
  col = bluered(length(seq(-max(abs(input.heatmap), na.rm = T), max(abs(input.heatmap), na.rm = T), by = 0.5)) - 1),
  scale = "none",
  key = TRUE,
  symkey = F,
  density.info = "none",
  trace = "none", 
  cexCol = 0.8, 
  cexRow = 0.7, 
  labRow = c(
    paste(TAX[rownames(pathogens.FL.heat), "genus"], rownames(pathogens.FL.heat), sep = "_"),
    "",
    paste(TAX[rownames(pathogens.PA.heat), "genus"], rownames(pathogens.PA.heat), sep = "_"),
    "",
    paste(TAX[rownames(pathogens.SED.heat), "genus"], rownames(pathogens.SED.heat), sep = "_")
  ),
  labCol = paste(ENV.PA$Island, "_", ENV.PA$Transect, "_", ENV.PA$Distance, "m", sep = "")[order(ENV.PA$Island)],
  keysize = 1,
  key.title = NA,
  key.par = list(mar = c(5, 2, 1, 0), cex = 0.6),
  key.xlab = "clr-transformed sequence counts",
  dendrogram = "none",
  Colv = F,
  Rowv = F,
  breaks = seq(-max(abs(input.heatmap), na.rm = T), max(abs(input.heatmap), na.rm = T), by = 0.5),
  lhei = c(1, 9)
)
segments(
  rep(0.09, 3),
  c(0.063, 0.222, 0.768),
  rep(0.09, 3),
  c(0.209, 0.756, 0.96),
  lwd = 1
)
text(
  rep(0.09, 3),
  apply(cbind(c(0.063, 0.222, 0.768), c(0.209, 0.756, 0.96)), 1, mean),
  labels = c("SED", "PA", "FL"),
  las = 3,
  pos = 2
)
text(
  0.18,
  0.768,
  labels = "No data",
  las = 3,
  pos = 3,
  cex = 0.8
)
dev.off()

### end pathogens ####